import java.util.*;
/* 
 * Links are chosen accordingly to a "copying" model (Kleinberg et al., '99,
 * http://www.cs.cornell.edu/home/kleinber/web-graph.ps)
 * 
 * Parameters of the model: 
 * the probability copyp of choosing to copy the links of a Blog
 * from a given set of Blogs chosen uniformly at random (u.a.r); with probability
 * 1-copyp, the outlink target is simply chosen u.a.r from the given set.  
 * 
 * Note that for simplicity, this policy returns only one target Blog 
 * (in the Vector returned by the choose() method). This was decided
 * upon in order to avoid the complexities involved in keeping a fixed out-degree
 * greater than 1 (one would need to have a set of "original" blogs and determine
 * what to do if one of these "original" Blogs with potentially smaller out-degree
 * were chose as the Blog to copy from, etc.)  
 */
public class CopyLinkPolicy1 extends LinkPolicy {
    
    private double copyp=0.5;   // probability that links are chosen by copying
    //   rather than choosing targets uniformly at
    //   random
    
    /** Constructor: a new CopyLinkPolicy with probability copyp of choosing
      * the target link by copying the link(s) of a Blog chosen uar.
      Precondition: 0.0<= copyp < 1.0*/
    public CopyLinkPolicy1(double copyp) {
        this.copyp= copyp;
    }
    
    /** = a Vector consisting of a single Blog among existing Blogs 
      * chosen to be the link target, according to the Copy model
      * If a Blog is chosen to copy from that does not have any outlinks,
      * that Blog itself is returned as the target. */
    public Vector<Blog> choose(Vector<Blog> candidates) {
        
        Vector<Blog> outVec= new Vector<Blog>(); // the output
        
        // pick a Blog to either link to or copy links of
        Blog blogChoice= LinkPolicy.BlogUAR(candidates); 
        
        
        if (coinComesUpHeads(copyp)) { // chosen to copy
            Vector<Blog> bcOutLinks= blogChoice.getOutLinks();
            
            // if blogChoice has no outlinks, simply link to it 
            if (bcOutLinks.size() == 0) {
                outVec.add(blogChoice);
            }
            
            // blogChoice has links to copy, so copy them
            // inv: i of the outlinks in bcOutLinks have been copied
            for (int i= 0; i< bcOutLinks.size(); i= i+1) {
                outVec.add(bcOutLinks.get(i));
            }
            // post: OutLinks.size() links have been copied
        }
        else { // blogChoice itself should be the target
            outVec.add(blogChoice);
        }
        return outVec;
    }
    
    /** test the copying model. 
      * Not in a testing framework because output is
      * probabilistic and so needs to be eyeballed */
    public static void testCopy(double copyp) {
        Blog x= new Blog(); Blog y=new Blog(); Blog z=new Blog(); 
        Vector<Blog> vec= new Vector<Blog>(); 
        vec.add(x); vec.add(y); vec.add(z);
        z.linkTo(x); z.linkTo(y);
        
        // pick targets for a new incoming Blog
        CopyLinkPolicy1 pol= new CopyLinkPolicy1(copyp);
        Vector<Blog> targets= pol.choose(vec);
        
        // create the new Blog
        Blog newbie= new Blog(targets);
        
        System.out.println("copy prob: " + copyp + "\nchoices: " + vec);
        System.out.println("newbie Blog: " + newbie);
        
    }
    
}